'==============================================================================
' Copyright  Micromega Corporation 2002-2008. All rights reserved.
'
' @file:	demo2.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point, displays the temperature in degrees Celsius and degrees
' Fahrenheit, and calculates the minimum and maximum temperatures.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	May 28, 2008
'	- corrected comments for DS1620 pins
'   March 31, 2008
'	- updated for ARMbasic V7
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3.1 definitions ---------------------------------

#include "FPUspi.bas"					' select either SPI or I2C
'#include "FPUi2c.bas"

'-------------------- DS1620 pin definitions ----------------------------------

#define SHIFTclkNEGATIVE 1
#include <shift.bas>

const	DS_DATA     = 10				' DS1620 data (1K to pin 1)
const	DS_CLK      = 11				' DS1620 clock (pin 2)
const	DS_RST		= 12				' DS1620 reset/enable (pin 3)

'-------------------- uM-FPU register definitions -----------------------------

const	DegC		= 8					' temperature in degrees Celsius
const	DegC_Min	= 9					' minimum temperature
const	DegC_Max	= 10				' maximum temperature
const	DegF		= 11				' temperature in degrees Fahrenheit
const	F1_8		= 12				' constant 1.8

'==============================================================================
'-------------------- subroutines ---------------------------------------------
'==============================================================================

'-------------------- Init_DS1620 ---------------------------------------------

SUB Init_DS1620
	IO(DS_RST) = 0						' initialize pin states
	IO(DS_CLK) = 1
	WAIT(100)

	OUT(DS_RST) = 1						' configure for CPU control
	shiftValues(0) = $0C
	shiftCounts(0) = 8
	shiftValues(1) = $02
	shiftCounts(1) = 8
	SHIFTOUT(DS_DATA, DS_CLK, 1, 2)
	OUT(DS_RST) = 0
	WAIT(100)

	OUT(DS_RST) = 1						' start temperature conversions
	shiftValues(0) = $EE
	shiftCounts(0) = 8
	SHIFTOUT(DS_DATA, DS_CLK, 1, 1)
	OUT(DS_RST) = 0
	WAIT(1000)							' wait for first conversion
END SUB

'-------------------- Read_DS1620 ---------------------------------------------

FUNCTION Read_DS1620
	dim n as integer
	OUT(DS_RST) = 1						' read temperature value
	shiftValues(0) = $AA
	shiftCounts(0) = 8
	SHIFTOUT(DS_DATA, DS_CLK, 1, 1)

	shiftCounts(0) = 9	
	SHIFTIN(DS_DATA, DS_CLK, 1, 1)
	n = shiftValues(0)
	OUT(DS_RST) = 0						' extend the sign bit
	if (n AND 0x100) then n = n OR 0xFFFFFF00
	return n
END FUNCTION

'-------------------- Read_Temp -----------------------------------------------

SUB Read_Temp
	dim rawTemp as integer
	rawTemp = Read_DS1620				' read temperature value										
	Fpu_Write3(SELECTA, DegC, LOADWORD)	' send rawTemp to uM-FPU
	Fpu_WriteWord(rawTemp)				' convert to floating point
	Fpu_Write3(FSET0, FDIVI, 2)			' divide by 2 to get degrees Celsius
END SUB

'-------------------- Print_TempC ---------------------------------------------
' input:  reg         selects the uM-FPU register to display
'------------------------------------------------------------------------------

SUB Print_TempC(reg)
	Fpu_Write2(SELECTA, reg)			' select reg as register A
	gosub Print_Float(81)				' print reg as formatted floating point
END SUB

'-------------------- Print_TempF ---------------------------------------------
' input:  reg         selects the uM-FPU register to convert and display
'------------------------------------------------------------------------------

SUB Print_TempF(reg)					' degF = reg * 1.8 + 32
	Fpu_Write8(SELECTA, DegF, FSET, reg, FMUL, F1_8, FADDI, 32)
	Print_Float(81)						' print reg as formatted floating point
END SUB

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Demo 2"
	print "------"

	if Fpu_Reset <> SYNC_CHAR then		' reset FPU and check synchronization
		print "uM-FPU not detected"
		end
	else
		Print_Version					' display FPU version number
		print
	endif

	Init_DS1620							' initialize DS1620
									
										' load floating point constant 1.8
	Fpu_Write3(SELECTA, F1_8, ATOF)
	Fpu_WriteString("1.8")
	Fpu_Write(FSET0)

	print "              Min   Current   Max"
	print "             -----  -------  -----"


	Read_Temp							' get initial temperature reading
										' initialize MIN/MAX values
	Fpu_Write4(SELECTA, DegC_Min, FSET, DegC)
	Fpu_Write4(SELECTA, DegC_Max, FSET, DegC)

'-------------------- main loop -----------------------------------------------

	do
		Read_Temp						' get temperature reading
										' calculate min/max values
		Fpu_Write4(SELECTA, DegC_Min, FMIN, DegC)
		Fpu_Write4(SELECTA, DegC_Max, FMAX, DegC)

		print "Celsius   ";				' display temperatures in Celsius
		Print_TempC(DegC_Min)
		Print_TempC(DegC)
		Print_TempC(DegC_Max)
		print

		print "Fahrenheit";				' display temperatures in Fahrenheit
		Print_TempF(DegC_Min)
		Print_TempF(DegC)
		Print_TempF(DegC_Max)
		print

		wait(2000)						' delay 2 seconds
		print
	loop

