'==============================================================================
' Copyright  Micromega Corporation 2002-2008. All rights reserved.
'
' @file: 	sensirion.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. It converts
' temperature and humidity readings from a Sensirion SHT1x sensor.
'
' @author Cam Thompson, Micromega Corporation
' @version
'   March 31, 2008
'	- updated for ARMbasic V7
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3.1 definitions ---------------------------------

#include "FPUspi.bas"					' select either SPI or I2C
'#include "FPUi2c.bas"

'-------------------- Sensirion SHT1x/SHT7x definitions------------------------

#define SHIFTpreSample 1
#include <shift.bas>

const	ShtClock	= 8					' Sensirion SHT1x clock (pin 3)
const	ShtData		= 9					' Sensirion SHT1x data (pin 1)

'-------------------- uM-FPU register definitions -----------------------------

const	RawTemp		= 10				' raw temperature
const	TempC		= 11				' temperature - Celsius
const	TempF		= 12				' temperature - Fahrenheit
const	RawHumidity	= 13				' raw humidity
const	RhLinear	= 14				' relative humidity - linear
const	RhTrue		= 15				' relative humidity - compensated
const	H			= 16				' H value for dew point calculation
const	DewPoint	= 17				' dew point

const	C2			= 20				' constants for RhLinear calculation
const	C3			= 21
const	T1			= 22				' constants for RhTrue calculation
const	T2			= 23
const	K1			= 24				' constant for DewPoint calculation 
const	K2			= 25
const	K3			= 26
const	K4			= 27

'==============================================================================
'-------------------- subroutines ---------------------------------------------
'==============================================================================

'-------------------- Sensirion SHT1x/SHT7x routines --------------------------

SUB SHT_Reset							' reset the Sensirion SHT1x
	shiftValues(0) = $1FF
	shiftCounts(0) = 9
	SHIFTOUT(ShtData, ShtClock, 0, 9)
END SUB

FUNCTION SHT_Write(cmd)
	dim ack as integer
	
	INPUT ShtData						' send start sequence
	LOW ShtClock
	HIGH ShtClock
	LOW ShtData
	OUTPUT ShtData
	LOW ShtClock
	HIGH ShtClock
	INPUT ShtData
	LOW ShtClock
										
	shiftValues(0) = cmd				' send command and get ACK
	shiftCounts(0) = 8
	SHIFTOUT(ShtData, ShtClock, 0, 1)
	
	ack = IO(ShtData)
	HIGH ShtClock
	LOW ShtClock
	return ack
END FUNCTION

FUNCTION SHT_Read(ack)					' read byte and send ACK
	shiftCounts(0) = 8	
	SHIFTIN(ShtData, ShtClock, 0, 1)
	IO(ShtData) = ack
	HIGH ShtClock
	LOW ShtClock
	INPUT ShtData
	return shiftValues(0)
END FUNCTION

FUNCTION SHT_ReadWord					' read 16-bit value
	dim n as integer
	n = SHT_Read(0) << 8
	return n or SHT_Read(1)
END FUNCTION

FUNCTION SHT_ReadStatus:				' read status byte
	SHT_Write($07)
	return SHT_Read(1)
END FUNCTION

SUB SHT_WriteStatus(status)				' write status byte
	SHT_Write($06)
	SHT_Write(status)
END SUB

SUB SHT_Wait							' wait for ready (~250 msec timeout)
	dim i as integer
	for i = 1 to 250
		if NOT in(ShtData) then exit
		wait(1)
	next
END SUB

SUB SHT_Temperature						' read temperature and send to uM-FPU
	dim i as integer
	for i = 1 to 100
		if not SHT_Write($03) then exit
		SHT_Reset						' if NAK, reset the device and retry
	next
	SHT_Wait
	Fpu_Write3(SELECTA, RawTemp, LOADWORD)
	Fpu_WriteWord(SHT_ReadWord)
	Fpu_Write(FSET0)
END SUB

SUB SHT_Humidity						' read humidity and send to uM-FPU
	dim i as integer
	for i = 1 to 100
		if not SHT_Write($05) then exit
		SHT_Reset						' if NAK, reset the device and retry
	next
	SHT_Wait
	Fpu_Write3(SELECTA, RawHumidity, LOADWORD)
	Fpu_WriteWord(SHT_ReadWord)
	Fpu_Write(FSET0)
END SUB

SUB SetConstant(reg, BYREF s as string)
	Fpu_Write3(SELECTA, reg, ATOF)
	Fpu_WriteString(s)
	Fpu_Write(FSET0)
ENDSUB

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Sensirion Demo"
	
	if Fpu_Reset <> SYNC_CHAR then		' reset FPU and check synchronization
		print "uM-FPU not detected"
		end
	else
		Print_Version					' display FPU version number
		print
	endif
	
	SHT_Reset							' reset the Sensirion sensor
											
	SetConstant(C2, "0.0405")			' store floating point constants
	SetConstant(C3, "-2.8e-6")
	SetConstant(T1, "0.01")
	SetConstant(T2, "0.00008")
	SetConstant(K1, "0.4343")
	SetConstant(K2, "17.62")
	SetConstant(K3, "243.12")

'-------------------- main loop -----------------------------------------------

	do	
		' read temperature
		'-----------------
		SHT_Temperature
		print "Raw Temperature:  ";
		Print_Float(60)
		print

		' TempC = -40.0 + 0.01 * rawTemp
		' TempC = RawTemp / 100.0 - 40.0
		'-------------------------------	
		Fpu_Write8(SELECTA, TempC, FSET, RawTemp, FDIVI, 100, FSUBI, 40)
		print "Degrees C:        ";
		Print_Float(62)
		print

		' TempF = TempC converted to Fahrenheit
		'--------------------------------------
		Fpu_Write6(SELECTA, TempF, FSET, TempC, FCNV, 1)
		print "Degrees F:        ";
		Print_Float(62)
		print

		' read humidity
		'--------------
		SHT_Humidity
		print "Raw Humidity:     ";
		Print_Float(60)
		print

		' RhLinear = -4.0 + 0.0405 * rawHumidity + (-2.8E-6) * rawHumidity^2
		' RhLinear = ((rawHumidity * C3) + C2) * rawHumidity) - 4
		'-------------------------------------------------------------------
		Fpu_Write6(SELECTA, RhLinear, FSET, RawHumidity, FMUL, C3)
		Fpu_Write6(FADD, C2, FMUL, rawHumidity, FSUBI, 4)	
		print "Humidity (linear):";
		Print_Float(62)
		print

		' RhTrue = (TempC - 25.0) * (0.01 + 0.00008 * RawHumidity) + RhLinear
		' RhTrue = (TempC - 25.0) * (T2 * RawHumidity + T1) + RhLinear
		'--------------------------------------------------------------------	
		Fpu_Write6(SELECTA, RhTrue, FSET, TempC, FSUBI, 25)
		Fpu_Write7(F_LEFT, FSET, T2, FMUL, RawHumidity, FADD, T1)	
		Fpu_Write4(F_RIGHT, FMUL0, FADD, RhLinear)
		print "Humidity (true):  ";
		gosub Print_Float(62)
		print

		' H = (log10(RhTrue) - 2.0) / 0.4343 + (17.62 * TempC) / (243.12 + TempC)
		' H = (log10(RhTrue) - 2.0) / K1 + (K2 * TempC) / (K3 + TempC)
		'------------------------------------------------------------------------
		Fpu_Write4(SELECTA, H, FSET, RhTrue)
		Fpu_Write5(LOG10, FSUBI, 2, FDIV, K1)	
		Fpu_Write5(F_LEFT, FSET, K2, FMUL, TempC)
		Fpu_Write7(F_LEFT, FSET, K3, FADD, TempC, F_RIGHT, FDIV0)
		Fpu_Write2(F_RIGHT, FADD0)

		' DewPoint = 243.12 * H / (17.62 - H)
		' DewPoint = K3 * H / (K2 - H)
		'------------------------------------------------------------------------
		Fpu_Write6(SELECTA, DewPoint, FSET, K3, FMUL, H)
		Fpu_Write7(F_LEFT, FSET, K2, FSUB, H, F_RIGHT, FDIV0)
		print "Dew Point:        ";
		gosub Print_Float(62)
		print

		wait(2000)
		print
	loop
